"""University departments.
"""
import copy
import grok
from zope.component import createObject
from zope.component.interfaces import IFactory
from zope.interface import implementedBy
from waeup.interfaces import IDepartment, ICourse
from waeup.utils.helpers import getName
from waeup.widgets.interfaces import ITableProvider
from waeup.viewlets import (MainArea, LeftSidebar, Index, FormWrapMixin,
                            Manage, AddCourse)

class Department(grok.Container):
    grok.implements(IDepartment)

    # A simple counter for ids.
    next_id = 0L
    
    def __init__(self,
                 title=u'Unnamed Department',
                 title_prefix=u'department',
                 code=u"NA", **kw):
        super(Department, self).__init__(**kw)
        self.title = title
        self.title_prefix = title_prefix
        self.code = code
        self.courses = createObject(u'waeup.CourseContainer')
        self.courses.__parent__ = self
        self.courses.__name__ = 'courses'
        self.certificates = createObject(u'waeup.CertificateContainer')
        self.certificates.__parent__ = self
        self.certificates.__name__ = 'certificates'

    def traverse(self, name):
        if name == 'courses':
            return self.courses
        elif name == 'certificates':
            return self.certificates
        return None
        

class DepartmentFactory(grok.GlobalUtility):
    """A factory for department containers.
    """
    grok.implements(IFactory)
    grok.name(u'waeup.Department')
    title = u"Create a new department.",
    description = u"This factory instantiates new department instances."

    def __call__(self, *args, **kw):
        return Department(*args, **kw)

    def getInterfaces(self):
        return implementedBy(Department)


class Content(grok.Viewlet):
    """A viewlet that wraps the `AddDepartmentForm`.
    """
    grok.viewletmanager(MainArea)
    grok.context(IDepartment)
    grok.view(Index)
    grok.require('waeup.manageUniversity')

    def getName(self):
        return getName(self.context)

    def getParentName(self):
        return getName(self.context.__parent__)

class CourseList(grok.Viewlet):
    grok.viewletmanager(MainArea)
    grok.context(IDepartment)
    grok.view(Index)
    grok.order(3)

    def update(self):
        self.tables = ITableProvider(self.context).getTables(view=self.view)
        self.coursetable = self.tables[0]
        self.certificatetable = self.tables[1]
        self.coursetable.need()


class ManageForm(grok.EditForm):
    """Manage the basic properties of a `Department` instance.
    """

    form_fields = grok.AutoFields(IDepartment)

    @property
    def label(self):
        # Set department name as form label
        return self.form_fields['title'].field.get(self.context)
    
    @grok.action('Save')
    def save(self, **data):
        self.applyData(self.context, **data)
        return
    
    @grok.action('Save and return')
    def saveAndReturn(self, **data):
        self.applyData(self.context, **data)
        self.redirect(self.url(self.context))
        return

    @grok.action('Cancel')
    def cancel(self, **data):
        self.redirect(self.url(self.context))
        return

class ManageDepartment(FormWrapMixin, grok.Viewlet):
    """A viewlet that renders the `ManageForm`.
    """
    grok.viewletmanager(MainArea)
    grok.context(IDepartment)
    grok.view(Manage)
    grok.require('waeup.manageUniversity')
    
    formview_name = 'manageform' # The name of the formview we want to
                                 # be rendered in this viewlet.

class ManageLink(grok.Viewlet):
    grok.viewletmanager(LeftSidebar)
    grok.context(IDepartment)
    grok.view(Index)
    grok.order(5)
    # This link is only displayed, if the user is allowed to use it!
    grok.require('waeup.manageUniversity')
    
    def render(self):
        return u'<div class="portlet"><a href="manage">Settings</a></div>'

class AddCourseForm(grok.AddForm):
    grok.context(IDepartment)
    form_fields = copy.deepcopy(grok.AutoFields(ICourse))
    form_fields.get('code').field.readonly = False
    label = 'Add a course'

    @grok.action('Add course')
    def addCourse(self, **data):
        course = createObject(u'waeup.Course')
        self.applyData(course, **data)
        try:
            self.context.courses.addCourse(course)
        except DuplicationError:
            self.status = Invalid('The name chosen already exists '
                                  'in the database')
            return
        self.redirect(self.url(self.context))

    @grok.action('Cancel')
    def cancel(self, **data):
        self.redirect(self.url(self.context))
        return

        
class AddCourse(FormWrapMixin, grok.Viewlet):
    """A viewlet that wraps the `AddFacultyForm`.
    """
    grok.viewletmanager(MainArea)
    grok.context(IDepartment)
    grok.view(AddCourse)
    grok.require('waeup.manageUniversity')

    formview_name = 'addcourseform' # The name of the formview we
                                    # want to be rendered in this
                                    # viewlet.
